@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2015 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2015 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -50,6 +50,29 @@ class RsyncListFilesFailure(Exception):
     pass
 
 
+class DataTransferFailure(Exception):
+    """
+    Used to pass rsync failure details
+    """
+
+    @classmethod
+    def from_rsync_error(cls, e, msg):
+        """
+        This method build a DataTransferFailure exception and report the
+        provided message to the user (both console and log file) along with
+        the output of the failed rsync command.
+
+        :param CommandFailedException e: The exception we are handling
+        :param str msg: a descriptive message on what we are trying to do
+        :return DataTransferFailure: will contain the message provided in msg
+        """
+        details = msg
+        details += "\nrsync error:\n"
+        details += e.args[0]['out']
+        details += e.args[0]['err']
+        return cls(details)
+
+
 class Command(object):
     """
     Simple wrapper for a shell command
@@ -221,12 +244,31 @@ class Rsync(Command):
             for path in exclude_and_protect:
                 options += ["--exclude=%s" % (path,), "--filter=P_%s" % (path,)]
         if args:
-            options += args
+            options += self._args_for_suse(args)
         if bwlimit is not None and bwlimit > 0:
             options += ["--bwlimit=%s" % bwlimit]
         Command.__init__(self, rsync, args=options, check=check,
                          allowed_retval=allowed_retval, **kwargs)
 
+    def _args_for_suse(self, args):
+        """
+        Mangle args for SUSE compatibility
+
+        See https://bugzilla.opensuse.org/show_bug.cgi?id=898513
+        """
+        # Prepend any argument starting with ':' with a space
+        # Workaround for SUSE rsync issue
+        return [' ' + a if a.startswith(':') else a for a in args]
+
+    def getoutput(self, *args, **kwargs):
+        """
+        Run the command and return the output and the error (if present)
+        """
+        # Prepares args for SUSE
+        args = self._args_for_suse(args)
+        # Invoke the base class method
+        return super(Rsync, self).getoutput(*args, **kwargs)
+
     def from_file_list(self, filelist, src, dst, *args, **kwargs):
         """
         This method copies filelist from src to dst.
@@ -420,9 +462,9 @@ class Rsync(Command):
                 # destination, rsync will discover the difference in any case.
                 # It is then safe to skip checksum check here.
                 dst_item = ref_hash.get(item.path, None)
-                if (dst_item is None
-                        or dst_item.size != item.size
-                        or dst_item.date != item.date):
+                if (dst_item is None or
+                        dst_item.size != item.size or
+                        dst_item.date != item.date):
                     safe_list.write(item.path + '\n')
                     continue
 
