@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2014 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2013-2015 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -20,6 +20,7 @@ import os
 import dateutil.parser
 import dateutil.tz
 import collections
+import logging
 from barman import xlog
 from barman.compression import identify_compression
 
@@ -27,6 +28,8 @@ from barman.compression import identify_compression
 # as property.
 Tablespace = collections.namedtuple('Tablespace', 'name oid location')
 
+_logger = logging.getLogger(__name__)
+
 
 def output_tablespace_list(tablespaces):
     """
@@ -57,6 +60,16 @@ def load_tablespace_list(string):
         return None
 
 
+def null_repr(obj):
+    """
+    Return the literal representation of an object
+
+    :param object obj: object to represent
+    :return str|None: Literal representation of an object or None
+    """
+    return repr(obj) if obj else None
+
+
 def load_datetime_tz(time_str):
     """
     Load datetime and ensure the result is timezone-aware.
@@ -141,6 +154,10 @@ class FieldListFile(object):
         the corresponding attribute. If a provided keyword argument doesn't
         has a corresponding attribute an AttributeError exception is raised.
 
+        The values provided to the constructor must be of the appropriate
+        type for the corresponding attribute.
+        The constructor will not attempt any validation or conversion on them.
+
         This class is meant to be an abstract base class.
 
         :raises: AttributeError
@@ -288,7 +305,6 @@ class WalFileInfo(FieldListFile):
     __slots__ = ()
 
     name = Field('name', doc='base name of WAL file')
-    full_path = Field('full_path', doc='complete path of the file')
     size = Field('size', load=int, doc='WAL file size after compression')
     time = Field('time', load=float, doc='WAL file modification time '
                                          '(seconds since epoch)')
@@ -309,7 +325,6 @@ class WalFileInfo(FieldListFile):
         """
         stat = os.stat(filename)
         kwargs.setdefault('name', os.path.basename(filename))
-        kwargs.setdefault('full_path', os.path.abspath(filename))
         kwargs.setdefault('size', stat.st_size)
         kwargs.setdefault('time', stat.st_mtime)
         if 'compression' not in kwargs:
@@ -330,11 +345,10 @@ class WalFileInfo(FieldListFile):
             self.compression)
 
     @classmethod
-    def from_xlogdb_line(cls, server, line):
+    def from_xlogdb_line(cls, line):
         """
         Parse a line from xlog catalogue
 
-        :param Server server: server object
         :param str line: a line in the wal database to parse
         :rtype: WalFileInfo
         """
@@ -350,12 +364,31 @@ class WalFileInfo(FieldListFile):
         # The to_xlogdb_line method writes None values as literal 'None'
         if compression == 'None':
             compression = None
-        full_path = server.get_wal_full_path(name)
         size = int(size)
         time = float(time)
-        return cls(name=name, full_path=full_path, size=size, time=time,
+        return cls(name=name, size=size, time=time,
                    compression=compression)
 
+    def to_json(self):
+        """
+        Return an equivalent dictionary that can be encoded in json
+        """
+        return dict(self.items())
+
+    def relpath(self):
+        """
+        Returns the WAL file path relative to the server's wals_directory
+        """
+        return os.path.join(xlog.hash_dir(self.name), self.name)
+
+    def fullpath(self, server):
+        """
+        Returns the WAL file full path
+
+        :param barman.server.Server server: the server that owns the wal file
+        """
+        return os.path.join(server.config.wals_directory, self.relpath())
+
 
 class UnknownBackupIdException(Exception):
     """
@@ -397,12 +430,13 @@ class BackupInfo(FieldListFile):
     begin_time = Field('begin_time', load=load_datetime_tz)
     begin_xlog = Field('begin_xlog')
     begin_wal = Field('begin_wal')
-    begin_offset = Field('begin_offset')
+    begin_offset = Field('begin_offset', load=int)
     size = Field('size', load=int)
+    deduplicated_size = Field('deduplicated_size', load=int)
     end_time = Field('end_time', load=load_datetime_tz)
     end_xlog = Field('end_xlog')
     end_wal = Field('end_wal')
-    end_offset = Field('end_offset')
+    end_offset = Field('end_offset', load=int)
     status = Field('status', default=EMPTY)
     server_name = Field('server_name')
     error = Field('error')
@@ -410,9 +444,12 @@ class BackupInfo(FieldListFile):
     config_file = Field('config_file')
     hba_file = Field('hba_file')
     ident_file = Field('ident_file')
-    backup_label = Field('backup_label', load=ast.literal_eval, dump=repr)
+    included_files = Field('included_files',
+                           load=ast.literal_eval, dump=null_repr)
+    backup_label = Field('backup_label', load=ast.literal_eval, dump=null_repr)
 
-    __slots__ = ('server', 'config', 'backup_manager', 'backup_id')
+    __slots__ = ('server', 'config', 'backup_manager',
+                 'backup_id', 'backup_version')
 
     def __init__(self, server, info_file=None, backup_id=None, **kwargs):
         # Initialises the attributes for the object based on the predefined keys
@@ -431,6 +468,8 @@ class BackupInfo(FieldListFile):
         self.server = server
         self.config = server.config
         self.backup_manager = self.server.backup_manager
+        self.server_name = self.config.name
+        self.mode = self.backup_manager.name
         if backup_id:
             # Cannot pass both info_file and backup_id
             if info_file:
@@ -439,15 +478,12 @@ class BackupInfo(FieldListFile):
             self.backup_id = backup_id
             self.filename = self.get_filename()
             # Check if a backup info file for a given server and a given ID
-            # already exists. If not, create it from scratch
-            if not os.path.exists(self.filename):
-                self.server_name = self.config.name
-                self.mode = self.backup_manager.name
-            else:
+            # already exists. If so load the values from the file.
+            if os.path.exists(self.filename):
                 self.load(filename=self.filename)
         elif info_file:
             if hasattr(info_file, 'read'):
-            # We have been given a file-like object
+                # We have been given a file-like object
                 self.load(file_object=info_file)
             else:
                 # Just a file name
@@ -456,6 +492,16 @@ class BackupInfo(FieldListFile):
         elif not info_file:
             raise BackupInfoBadInitialisation(
                 'backup_id and info_file parameters are both unset')
+        # Manage backup version for new backup structure
+        self.backup_version = 2
+        try:
+            # the presence of pgdata directory is the marker of version 1
+            if self.backup_id is not None and os.path.exists(
+                    os.path.join(self.get_basebackup_directory(), 'pgdata')):
+                self.backup_version = 1
+        except Exception as e:
+            _logger.warning("Error detecting backup_version, use default: 2.\n "
+                            "Failure reason: %s", e)
 
     def get_required_wal_segments(self):
         """
@@ -478,8 +524,10 @@ class BackupInfo(FieldListFile):
             for x in self.get_required_wal_segments():
                 yield self.server.get_wal_full_path(x)
         if target in ('wal', 'full'):
-            for wal_info in self.server.get_wal_until_next_backup(self):
-                yield wal_info.full_path
+            for wal_info in self.server.get_wal_until_next_backup(
+                    self,
+                    include_history=True):
+                yield wal_info.fullpath(self.server)
 
     def detect_backup_id(self):
         """
@@ -498,6 +546,42 @@ class BackupInfo(FieldListFile):
         return os.path.join(self.config.basebackups_directory,
                             self.backup_id)
 
+    def get_data_directory(self, tablespace_oid=None):
+        """
+        Get path to the backup data dir according with the backup version
+
+        If tablespace_oid is passed, build the path to the tablespace
+        base directory, according with the backup version
+
+        :param str tablespace_oid: the oid of a valid tablespace
+        """
+        # Check if a tablespace oid is passed and if is a valid oid
+        if tablespace_oid is not None and (
+                self.tablespaces is None or
+                all(str(tablespace_oid) != str(tablespace.oid)
+                    for tablespace in self.tablespaces)):
+            raise ValueError("Invalid tablespace OID %s" % tablespace_oid)
+        # Build the requested path according to backup_version value
+        path = [self.get_basebackup_directory()]
+        # Check te version of the backup
+        if self.backup_version == 2:
+            # If an oid has been provided, we are looking for a tablespace
+            if tablespace_oid is not None:
+                # Append the oid to the basedir of the backup
+                path.append(str(tablespace_oid))
+            else:
+                # Looking for the data dir
+                path.append('data')
+        else:
+            # Backup v1, use pgdata as base
+            path.append('pgdata')
+            # If a oid has been provided, we are looking for a tablespace.
+            if tablespace_oid is not None:
+                # Append the path to pg_tblspc/oid folder inside pgdata
+                path.extend(('pg_tblspc', str(tablespace_oid)))
+        # Return the built path
+        return os.path.join(*path)
+
     def get_filename(self):
         """
         Get the default filename for the backup.info file based on
@@ -530,3 +614,39 @@ class BackupInfo(FieldListFile):
         result.update(backup_id=self.backup_id, server_name=self.server_name,
                       mode=self.mode, tablespaces=self.tablespaces)
         return result
+
+    def to_json(self):
+        """
+        Return an equivalent dictionary that uses only json-supported types
+        """
+        data = self.to_dict()
+        # Convert fields which need special types not supported by json
+        if data.get('tablespaces') is not None:
+            data['tablespaces'] = [list(item)
+                                   for item in data['tablespaces']]
+        if data.get('begin_time') is not None:
+            data['begin_time'] = data['begin_time'].ctime()
+        if data.get('end_time') is not None:
+            data['end_time'] = data['end_time'].ctime()
+        return data
+
+    @classmethod
+    def from_json(cls, server, json_backup_info):
+        """
+        Factory method that builds a BackupInfo object
+        from a json dictionary
+
+        :param barman.Server server: the server related to the Backup
+        :param dict json_backup_info: the data set containing values from json
+        """
+        data = dict(json_backup_info)
+        # Convert fields which need special types not supported by json
+        if data.get('tablespaces') is not None:
+            data['tablespaces'] = [Tablespace._make(item)
+                                   for item in data['tablespaces']]
+        if data.get('begin_time') is not None:
+            data['begin_time'] = load_datetime_tz(data['begin_time'])
+        if data.get('end_time') is not None:
+            data['end_time'] = load_datetime_tz(data['end_time'])
+        # Instantiate a BackupInfo object using the converted fields
+        return cls(server, **data)
