@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2015 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2013-2015 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -125,6 +125,7 @@ def _dispatch(obj, prefix, name, *args, **kwargs):
         raise ValueError("The object %r does not have the %r method" % (
             obj, method_name))
 
+
 def is_quiet():
     """
     Calls the "is_quiet" method, accessing the protected parameter _quiet
@@ -133,6 +134,7 @@ def is_quiet():
     """
     return _writer.is_quiet()
 
+
 def is_debug():
     """
     Calls the "is_debug" method, accessing the protected parameter _debug
@@ -141,6 +143,7 @@ def is_debug():
     """
     return _writer.is_debug()
 
+
 def debug(message, *args, **kwargs):
     """
     Output a message with severity 'DEBUG'
@@ -199,14 +202,14 @@ def exception(message, *args, **kwargs):
     """
     # ignore and raise_exception are keyword-only parameters
     ignore = kwargs.pop('ignore', False)
-    #noinspection PyNoneFunctionAssignment
+    # noinspection PyNoneFunctionAssignment
     raise_exception = kwargs.pop('raise_exception', None)
     if not ignore:
         kwargs.setdefault('is_error', True)
     _put('exception', message, *args, **kwargs)
     if raise_exception:
         if callable(raise_exception):
-            #noinspection PyCallingNonCallable
+            # noinspection PyCallingNonCallable
             raise raise_exception(message)
         elif isinstance(raise_exception, BaseException):
             raise raise_exception
@@ -314,6 +317,9 @@ class ConsoleOutputWriter(object):
         #: values.
         self.minimal = False
 
+        #: The server is active
+        self.active = True
+
     def _out(self, message, args):
         """
         Print a message on standard output
@@ -395,6 +401,58 @@ class ConsoleOutputWriter(object):
         """
         # TODO: evaluate to display something useful here
 
+    def result_recovery(self, results):
+        """
+        Render the result of a recovery.
+
+        """
+        if len(results['changes']) > 0:
+            self.info("")
+            self.info("IMPORTANT")
+            self.info("These settings have been modified to prevent "
+                      "data losses")
+            self.info("")
+
+            for assertion in results['changes']:
+                self.info("%s line %s: %s = %s",
+                          assertion.filename,
+                          assertion.line,
+                          assertion.key,
+                          assertion.value)
+
+        if len(results['warnings']) > 0:
+            self.info("")
+            self.info("WARNING")
+            self.info("You are required to review the following options"
+                      " as potentially dangerous")
+            self.info("")
+
+            for assertion in results['warnings']:
+                self.info("%s line %s: %s = %s",
+                          assertion.filename,
+                          assertion.line,
+                          assertion.key,
+                          assertion.value)
+
+        if results['delete_barman_xlog']:
+            self.info("")
+            self.info("After the recovery, please remember to remove the "
+                      "\"barman_xlog\" directory")
+            self.info("inside the PostgreSQL data directory.")
+
+        if results['get_wal']:
+            self.info("")
+            self.info("WARNING: 'get-wal' is in the specified "
+                      "'recovery_options'.")
+            self.info("Before you start up the PostgreSQL server, please "
+                      "review the recovery.conf file")
+            self.info("inside the target directory. Make sure that "
+                      "'restore_command' can be executed by "
+                      "the PostgreSQL user.")
+        self.info("")
+        self.info("Your PostgreSQL server has been successfully "
+                  "prepared for recovery!")
+
     def _record_check(self, server_name, check, status, hint):
         """
         Record the check line in result_check_map attribute
@@ -408,17 +466,19 @@ class ConsoleOutputWriter(object):
         """
         self.result_check_list.append(dict(
             server_name=server_name, check=check, status=status, hint=hint))
-        if not status:
+        if not status and self.active:
             global error_occurred
             error_occurred = True
 
-    def init_check(self, server_name):
+    def init_check(self, server_name, active):
         """
         Init the check command
 
         :param str server_name: the server we are start listing
+        :param boolean active: The server is active
         """
         self.info("Server %s:" % server_name)
+        self.active = active
 
     def result_check(self, server_name, check, status, hint=None):
         """
@@ -556,14 +616,14 @@ class ConsoleOutputWriter(object):
             self.info("")
             self.info("  Catalog information:")
             self.info("    Retention Policy     : %s",
-                      data['retention_policy_status']
-                      or 'not enforced')
+                      data['retention_policy_status'] or
+                      'not enforced')
             self.info("    Previous Backup      : %s",
-                      data.setdefault('previous_backup_id', 'not available')
-                      or '- (this is the oldest base backup)')
+                      data.setdefault('previous_backup_id', 'not available') or
+                      '- (this is the oldest base backup)')
             self.info("    Next Backup          : %s",
-                      data.setdefault('next_backup_id', 'not available')
-                      or '- (this is the latest base backup)')
+                      data.setdefault('next_backup_id', 'not available') or
+                      '- (this is the latest base backup)')
         else:
             if data['error']:
                 self.info("  Error:            : %s",
@@ -657,6 +717,9 @@ class NagiosOutputWriter(ConsoleOutputWriter):
 
         Also set the exit code as 2 (CRITICAL) in case of errors
         """
+
+        global error_occurred, error_exit_code
+
         # List of all servers that have been checked
         servers = []
         # List of servers reporting issues
@@ -668,6 +731,13 @@ class NagiosOutputWriter(ConsoleOutputWriter):
             # Keep track of the servers with issues
             if not item['status'] and item['server_name'] not in issues:
                 issues.append(item['server_name'])
+
+        # Global error (detected at configuration level)
+        if len(issues) == 0 and error_occurred:
+            print "BARMAN CRITICAL - Global configuration errors"
+            error_exit_code = 2
+            return
+
         if len(issues) > 0:
             fail_summary = []
             details = []
@@ -711,7 +781,6 @@ class NagiosOutputWriter(ConsoleOutputWriter):
             # add the detailed list to the output
             for issue in details:
                 print issue
-            global error_exit_code
             error_exit_code = 2
         else:
             # No issues, all good!
