@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2015 2ndQuadrant Italia (Devise.IT S.r.L.)
+# Copyright (C) 2011-2015 2ndQuadrant Italia Srl
 #
 # This file is part of Barman.
 #
@@ -19,13 +19,14 @@
 This module contains utility functions used in Barman.
 """
 
+import datetime
+import errno
+import grp
+import json
 import logging
 import logging.handlers
 import os
 import pwd
-import grp
-import json
-import datetime
 
 
 _logger = logging.getLogger(__name__)
@@ -120,9 +121,9 @@ def pretty_size(size, unit=1024):
     """
     This function returns a pretty representation of a size value
 
-    :param int,long,float size: the number to to prettify
+    :param int|long|float size: the number to to prettify
     :param int unit: 1000 or 1024 (the default)
-    :rtype : str
+    :rtype: str
     """
     suffixes = ["B"] + [i + {1000: "B", 1024: "iB"}[unit] for i in "KMGTPEZY"]
     if unit == 1000:
@@ -228,4 +229,22 @@ class BarmanEncoder(json.JSONEncoder):
         if hasattr(obj, 'decode') and callable(obj.decode):
             return obj.decode('utf-8', 'replace')
         # Let the base class default method raise the TypeError
-        return super(BarmanEncoder, self).default(obj)
\ No newline at end of file
+        return super(BarmanEncoder, self).default(obj)
+
+
+def fsync_dir(dir_path):
+    """
+    Execute fsync on a directory ensuring it is synced to disk
+
+    :param str dir_path: The directory to sync
+    :raise OSError: If fail opening the directory
+    """
+    dir_fd = os.open(dir_path, os.O_DIRECTORY)
+    try:
+        os.fsync(dir_fd)
+    except OSError, e:
+        # On some filesystem doing a fsync on a directory
+        # raises an EINVAL error. Ignoring it is usually safe.
+        if e.errno != errno.EINVAL:
+            raise
+    os.close(dir_fd)
